@@ -18,7 +18,7 @@
  */
 package blbutil;
 
-import beagleutil.Samples;
+import vcf.Samples;
 
 /**
  * <p>An iterator for records in a file.  Each records contains
