@@ -249,10 +249,7 @@ public class Utilities {
      * @throws NullPointerException if {@code e == null}
      */
     public static void exit(Throwable t) {
-        System.out.print("ERROR: ");
-        System.out.println(t);
         t.printStackTrace(System.out);
-        System.out.println("terminating program.");
         System.exit(1);
     }
 
