@@ -50,12 +50,15 @@ public final class Par {
     private final int burnin;
     private final int iterations;
     private final int phase_states;
-    private final float phase_segment;
+    private final float phase_step;
+    private final float rare;
 
     // imputation parameters
     private final boolean impute;
     private final int imp_states;
     private final float imp_segment;
+    private final float imp_step;
+    private final int imp_nsteps;
     private final float cluster;
     private final boolean ap;
     private final boolean gp;
@@ -67,36 +70,37 @@ public final class Par {
     private final float overlap;
     private final long seed;
     private final int nthreads;
-    private final float step;
-    private final int nsteps;
+    private final float buffer;
 
     // undocumented parameters
     private final File truth;
 
     // default phasing parameters
-
     private static final int D_BURNIN = 6;
     private static final int D_ITERATIONS = 12;
     private static final int D_PHASE_STATES = 280;
-    private static final float D_PHASE_SEGMENT = 4.0f;
+    private static final float D_PHASE_STEP = 0.006f;
+    private static final float D_RARE = 0.0015f;
 
     // default imputation parameters
     private static final boolean D_IMPUTE = true;
     private static final int D_IMP_STATES = 1600;
     private static final float D_IMP_SEGMENT = 6.0f;
+    private static final float D_IMP_STEP = 0.1f;
+    private static final int D_IMP_NSTEPS = 7;
     private static final float D_CLUSTER = 0.005f;
     private static final boolean D_AP = false;
     private static final boolean D_GP = false;
 
     // default general parameters
     private static final int D_NE = 1_000_000;
-    private static final float D_ERR = 0.0001f;
+    private static final float D_ERR = -Float.MIN_VALUE;
     private static final float D_WINDOW = 40.0f;
     private static final float D_OVERLAP = 4.0f;
     private static final int D_SEED = -99999;
     private static final int D_NTHREADS = Integer.MAX_VALUE;
-    private static final float D_STEP = 0.1f;
-    private static final int D_NSTEPS = 7;
+    private static final float D_BUFFER = 0.6f;
+
 
     /**
      * Constructs a new {@code Parameters} instance from the specified
@@ -137,25 +141,27 @@ public final class Par {
         burnin = Validate.intArg("burnin", argsMap, false, D_BURNIN, 1, IMAX);
         iterations = Validate.intArg("iterations", argsMap, false, D_ITERATIONS, 1, IMAX);
         phase_states = Validate.intArg("phase-states", argsMap, false, D_PHASE_STATES, 1, IMAX);
-        phase_segment = Validate.floatArg("phase-segment", argsMap, false, D_PHASE_SEGMENT, FMIN, FMAX);
+        phase_step = Validate.floatArg("phase-step", argsMap, false, D_PHASE_STEP, FMIN, FMAX);
+        rare = Validate.floatArg("rare", argsMap, false, D_RARE, FMIN, 0.5f);
 
         // imputation parameters
         impute = Validate.booleanArg("impute", argsMap, false, D_IMPUTE);
         imp_states = Validate.intArg("imp-states", argsMap, false, D_IMP_STATES, 1, IMAX);
         imp_segment = Validate.floatArg("imp-segment", argsMap, false, D_IMP_SEGMENT, FMIN, FMAX);
+        imp_step = Validate.floatArg("imp-step", argsMap, false, D_IMP_STEP, FMIN, FMAX);
+        imp_nsteps = Validate.intArg("imp-nsteps", argsMap, false, D_IMP_NSTEPS, 1, IMAX);
         cluster = Validate.floatArg("cluster", argsMap, false, D_CLUSTER, 0.0f, FMAX);
         ap = Validate.booleanArg("ap", argsMap, D_AP, false);
         gp = Validate.booleanArg("gp", argsMap, D_GP, false);
 
         // general parameters
         ne = Validate.floatArg("ne", argsMap, false, D_NE, FMIN, FMAX);
-        err = Validate.floatArg("err", argsMap, false, D_ERR, 1e-10f, FMAX);
+        err = Validate.floatArg("err", argsMap, false, D_ERR, -FMIN, FMAX);
         window = Validate.floatArg("window", argsMap, false, D_WINDOW, FMIN, IMAX);
         overlap = Validate.floatArg("overlap", argsMap, false, D_OVERLAP, FMIN, IMAX);
+        buffer = Validate.floatArg("overlap", argsMap, false, D_BUFFER, FMIN, IMAX);
         seed = Validate.longArg("seed", argsMap, false, D_SEED, LMIN, LMAX);
         nthreads = modNthreads(Validate.intArg("nthreads", argsMap, false, D_NTHREADS, 1, IMAX));
-        step = Validate.floatArg("step", argsMap, false, D_STEP, FMIN, FMAX);
-        nsteps = Validate.intArg("nsteps", argsMap, false, D_NSTEPS, 2, IMAX);
 
         // undocumented parameters
         truth =  Validate.getFile(Validate.stringArg("truth", argsMap, false, null, null));
@@ -192,26 +198,26 @@ public final class Par {
                 + "phasing parameters ..." + nl
                 + "  burnin=<number of burnin iterations>               (default=" + D_BURNIN + ")" + nl
                 + "  iterations=<number of phasing iterations>          (default=" + D_ITERATIONS + ")" + nl
-                + "  phase-states=<model states for phasing>            (default=" + D_PHASE_STATES + ")" + nl
-                + "  phase-segment=<min haplotype segment length (cM)>  (default=" + D_PHASE_SEGMENT + ")" + nl + nl
+                + "  phase-states=<model states for phasing>            (default=" + D_PHASE_STATES + ")" + nl + nl
 
                 + "imputation parameters ..." + nl
                 + "  impute=<impute ungenotyped markers (true/false)>   (default=" + D_IMPUTE + ")" + nl
                 + "  imp-states=<model states for imputation>           (default=" + D_IMP_STATES + ")" + nl
                 + "  imp-segment=<min haplotype segment length (cM)>    (default=" + D_IMP_SEGMENT + ")" + nl
-                + "  imp-cluster=<max cM in a marker cluster>           (default=" + D_CLUSTER + ")" + nl
-                + "  imp-ap=<print posterior allele probabilities>      (default=" + D_AP + ")" + nl
-                + "  imp-gp=<print posterior genotype probabilities>    (default=" + D_GP + ")" + nl + nl
+                + "  imp-step=<IBS step length (cM)>                    (default=" + D_IMP_STEP + ")" + nl
+                + "  imp-nsteps=<number of IBS steps>                   (default=" + D_IMP_NSTEPS + ")" + nl
+                + "  cluster=<max cM in a marker cluster>               (default=" + D_CLUSTER + ")" + nl
+                + "  ap=<print posterior allele probabilities>          (default=" + D_AP + ")" + nl
+                + "  gp=<print posterior genotype probabilities>        (default=" + D_GP + ")" + nl + nl
 
                 + "general parameters ..." + nl
                 + "  ne=<effective population size>                     (default=" + D_NE + ")" + nl
-                + "  err=<allele mismatch rate>                         (default=" + D_ERR + ")" + nl
+                + "  err=<allele mismatch rate>                         (default: data dependent)" + nl
                 + "  window=<window length in cM>                       (default=" + D_WINDOW + ")" + nl
                 + "  overlap=<window overlap in cM>                     (default=" + D_OVERLAP + ")" + nl
                 + "  seed=<random seed>                                 (default=" + D_SEED + ")" + nl
-                + "  nthreads=<number of threads>                       (default: machine-dependent)" + nl
-                + "  step=<IBS step length (cM)>                        (default=" + D_STEP + ")" + nl
-                + "  nsteps=<number of IBS steps>                       (default=" + D_NSTEPS + ")" + nl + nl;
+                + "  nthreads=<number of threads>                       (default: machine dependent)" + nl + nl;
+
     }
 
     private static ChromInterval parseChromInt(String str) {
@@ -275,7 +281,7 @@ public final class Par {
      * if no ped parameter was specified
      */
     public File ped() {
-        return null; // ignoring ped file at present
+        return null;
     }
 
     /**
@@ -345,11 +351,30 @@ public final class Par {
     }
 
     /**
-     * Returns the phase-segment parameter.
-     * @return the phase-segment parameter
+     * Return the ratio of the phase-states parameter and the default
+     * phase-states parameter
+     * @return the ratio of the phase-states parameter and the default
+     * phase-states parameter
      */
-    public float phase_segment() {
-        return phase_segment;
+    public float scaleFactor() {
+        return (phase_states==D_PHASE_STATES) ? 1.0f
+                : (float) phase_states / D_PHASE_STATES;
+    }
+
+    /**
+     * Returns the phase-step parameter.
+     * @return the phase-step parameter
+     */
+    public float phase_step() {
+        return phase_step;
+    }
+
+    /**
+     * Returns the rare parameter
+     * @return the rare parameter
+     */
+    public float rare() {
+        return rare;
     }
 
     // imputation parameters
@@ -378,6 +403,22 @@ public final class Par {
         return imp_segment;
     }
 
+    /**
+     * Returns the imp-step parameter.
+     * @return the imp-step parameter
+     */
+    public float imp_step() {
+        return imp_step;
+    }
+
+    /**
+     * Returns the imp-nsteps parameter.
+     * @return the imp-nsteps parameter
+     */
+    public int imp_nsteps() {
+        return imp_nsteps;
+    }
+
     /**
      * Returns the cluster parameter.
      * @return the cluster parameter
@@ -413,11 +454,31 @@ public final class Par {
     }
 
     /**
-     * Returns the err parameter.
-     * @return the err parameter
+     * Returns the allele mismatch parameter for the specified number of samples.
+     * @param nHaps the number of reference and target haplotypes
+     * @return the allele mismatch parameter for the specified number of samples
      */
-    public float err() {
-        return err;
+    public float err(int nHaps) {
+        if (nHaps <= 0) {
+            throw new IllegalArgumentException(String.valueOf(nHaps));
+        }
+        return (err>=0) ? err : liStephensErr(nHaps);
+    }
+
+    /**
+     * Return an approximation to the allele mismatch probability suggested by
+     * Li and Stephens.  The approximation uses a Riemann sum approximation
+     * of the natural log function.
+     * Refs:
+     * Li N, Stephens M. Genetics 2003 Dec;165(4):2213-33
+     * Marchini J, Howie B. Myers S, McVean G, Donnelly P. 2007:39(7)906-13.
+     *
+     * @param nHaps the number of haplotypes
+     * @return the allele mismatch probability suggested by Li and Stepehens
+     */
+    private static float liStephensErr(int nHaps) {
+        double theta = 1/((Math.log(nHaps) + 0.5));
+        return (float) (theta/(2*(theta + nHaps)));
     }
 
     /**
@@ -436,6 +497,14 @@ public final class Par {
         return overlap;
     }
 
+    /**
+     * Return the buffer parameter.
+     * @return the buffer parameter.
+     */
+    public float buffer() {
+        return buffer;
+    }
+
     /**
      * Returns the seed parameter.
      * @return the seed parameter
@@ -452,22 +521,6 @@ public final class Par {
         return nthreads;
     }
 
-    /**
-     * Returns the step parameter.
-     * @return the step parameter
-     */
-    public float step() {
-        return step;
-    }
-
-    /**
-     * Returns the nsteps parameter.
-     * @return the nsteps parameter
-     */
-    public int nsteps() {
-        return nsteps;
-    }
-
     // undocumented parameters
 
     /**
