@@ -247,12 +247,13 @@ public class BasicMarker implements Marker {
         for (int j=0; j<5; ++j) {
             fields[j] = new String(fields[j]);
         }
+        String infoField = fields[7];
         this.chromIndex = extractChrom(fields[0], vcfRecord);
         this.pos = extractPos(fields[1], vcfRecord);
         this.ids = extractIds(chromIndex, pos, fields[2]);
         this.alleles = extractAlleles(chromIndex, pos, fields[3], fields[4]);
         this.nGenotypes = alleles.length*(alleles.length+1)/2;
-        this.end = extractEnd(chromIndex, pos, vcfRecord);
+        this.end = extractEnd(chromIndex, pos, infoField);
     }
 
     private static int extractChrom(String chrom, String vcfRecord) {
